@@ -8,7 +8,7 @@ import sys
 import tarfile
 import time
 import zipfile
-from glob import glob
+from glob import glob, iglob
 import urllib.request
 
 def remove_prefix(text):
@@ -69,10 +69,6 @@ class ReportHook():
             status += "        \r"
             sys.stderr.write(status)
 
-# set the location of the database download url
-DROPBOX_DATABASE_DOWNLOAD = "https://www.dropbox.com/sh/7qze7m7g9fe2xjg/AADHWzATSQcI0CNFD0sk7MAga"
-ZENODO_DATABASE_DOWNLOAD = "https://zenodo.org/record/3957592"
-
 def download(url, download_file, force=False):
     """
     Download a file from a url
@@ -109,13 +105,9 @@ def download_unpack_tar(download_file_name, folder, bowtie2_build, nproc, use_ze
     tar_file = os.path.join(folder, download_file_name + ".tar")
     md5_file = os.path.join(folder, download_file_name + ".md5")
 
-    #Download the list of all the files in the Dropbox folder
-    if not use_zenodo:
-        url_tar_file = "http://cmprod1.cibio.unitn.it/biobakery3/metaphlan_databases/{}.tar".format(download_file_name)
-        url_md5_file = "http://cmprod1.cibio.unitn.it/biobakery3/metaphlan_databases/{}.md5".format(download_file_name)
-    else:
-        url_tar_file = "https://zenodo.org/record/3957592/files/{}.tar?download=1".format(download_file_name)
-        url_md5_file = "https://zenodo.org/record/3957592/files/{}.md5?download=1".format(download_file_name)
+    #Download the list of all the files in the FPT    
+    url_tar_file = "http://cmprod1.cibio.unitn.it/biobakery4/metaphlan_databases/{}.tar".format(download_file_name)
+    url_md5_file = "http://cmprod1.cibio.unitn.it/biobakery4/metaphlan_databases/{}.md5".format(download_file_name)
 
     # download tar and MD5 checksum
     download(url_tar_file, tar_file)
@@ -160,19 +152,29 @@ def download_unpack_tar(download_file_name, folder, bowtie2_build, nproc, use_ze
         sys.stderr.write("Warning: Unable to extract {}.\n".format(tar_file))
 
     # uncompress sequences
-    bz2_file = os.path.join(folder, download_file_name + ".fna.bz2")
+    for bz2_file in iglob(os.path.join(folder, download_file_name + "_*.fna.bz2")):
+        fna_file = bz2_file[:-4]
+
+        if not os.path.isfile(fna_file):
+            sys.stderr.write('\n\nDecompressing {} into {}\n'.format(bz2_file, fna_file))
+
+            with open(fna_file, 'wb') as fna_h, \
+                bz2.BZ2File(bz2_file, 'rb') as bz2_h:
+                for data in iter(lambda: bz2_h.read(100 * 1024), b''):
+                    fna_h.write(data)
+        os.remove(bz2_file)
+
+    # join fasta files
+    sys.stderr.write('\n\nJoining FASTA databases\n')
+    with open(os.path.join(folder, download_file_name + ".fna"), 'w') as fna_h:
+        for fna_file in iglob(os.path.join(folder, download_file_name + "_*.fna")):
+            with open(fna_file, 'r') as fna_r:
+                for line in fna_r:
+                    fna_h.write(line)
     fna_file = os.path.join(folder, download_file_name + ".fna")
 
-    if not os.path.isfile(fna_file):
-        sys.stderr.write('\n\nDecompressing {} into {}\n'.format(bz2_file, fna_file))
-
-        with open(fna_file, 'wb') as fna_h, \
-            bz2.BZ2File(bz2_file, 'rb') as bz2_h:
-            for data in iter(lambda: bz2_h.read(100 * 1024), b''):
-                fna_h.write(data)
-
     # build bowtie2 indexes
-    if not glob(os.path.join(folder, download_file_name + "*.bt2")):
+    if not glob(os.path.join(folder, download_file_name + "*.bt2l")):
         bt2_base = os.path.join(folder, download_file_name)
         bt2_cmd = [bowtie2_build, '--quiet']
 
@@ -192,12 +194,20 @@ def download_unpack_tar(download_file_name, folder, bowtie2_build, nproc, use_ze
             sys.stderr.write("Fatal error running '{}'\nError message: '{}'\n\n".format(' '.join(bt2_cmd), e))
             sys.exit(1)
 
-    for bt2 in glob(os.path.join(folder, download_file_name + "*.bt2")):
-        os.chmod(bt2, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IROTH)  # change permissions to 664
+    try:
+        for bt2 in glob(os.path.join(folder, download_file_name + "*.bt2l")):
+            os.chmod(bt2, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IROTH)  # change permissions to 664
+    except PermissionError as e:
+        sys.stderr.write('Cannot change permission for {}. Make sure the files are readable.'.format(os.path.join(folder, download_file_name + "*.bt2l")))
 
-    sys.stderr.write('Removing uncompress database {}\n'.format(fna_file))
+    sys.stderr.write('Removing uncompressed databases\n')
     os.remove(fna_file)
 
+    # remove all the individual FASTA files but ViralDB
+    for fna_file in iglob(os.path.join(folder, download_file_name + "_*.fna")):
+        if not fna_file.endswith('_VSG.fna'):
+            os.remove(fna_file)
+    
 def download_unpack_zip(url,download_file_name,folder,software_name):
     """
     Download the url to the file and decompress into the folder
@@ -228,14 +238,17 @@ def download_unpack_zip(url,download_file_name,folder,software_name):
         except EnvironmentError:
             print("WARNING: Unable to remove the temp download: " + download_file)
 
-def resolve_latest_database(bowtie2_db,mpa_latest_url, force=False):
-    if os.path.exists(os.path.join(bowtie2_db,'mpa_latest')):
+def resolve_latest_database(bowtie2_db,mpa_latest_url, force=False, offline=False):
+    if not offline and os.path.exists(os.path.join(bowtie2_db,'mpa_latest')):
         ctime_latest_db = int(os.path.getctime(os.path.join(bowtie2_db,'mpa_latest')))
         if int(time.time()) - ctime_latest_db > 31536000:         #1 year in epoch
             os.rename(os.path.join(bowtie2_db,'mpa_latest'),os.path.join(bowtie2_db,'mpa_previous'))
             download(mpa_latest_url, os.path.join(bowtie2_db,'mpa_latest'), force=True)
 
     if not os.path.exists(os.path.join(bowtie2_db,'mpa_latest') or force):
+        if offline:
+            print("Database cannot be downloaded with the --offline option activated")
+            sys.exit()        
         download(mpa_latest_url, os.path.join(bowtie2_db,'mpa_latest'))
 
     with open(os.path.join(bowtie2_db,'mpa_latest')) as mpa_latest:
@@ -243,7 +256,7 @@ def resolve_latest_database(bowtie2_db,mpa_latest_url, force=False):
     
     return ''.join(latest_db_version)
 
-def check_and_install_database(index, bowtie2_db, bowtie2_build, nproc, force_redownload_latest):
+def check_and_install_database(index, bowtie2_db, bowtie2_build, nproc, force_redownload_latest, offline):
     # Create the folder if it does not already exist
     if not os.path.isdir(bowtie2_db):
         try:
@@ -256,8 +269,9 @@ def check_and_install_database(index, bowtie2_db, bowtie2_build, nproc, force_re
     
     use_zenodo = False
     try:
-        if urllib.request.urlopen("http://cmprod1.cibio.unitn.it/biobakery3/metaphlan_databases/mpa_latest").getcode() != 200:
-            use_zenodo = True
+        if not offline and urllib.request.urlopen("http://cmprod1.cibio.unitn.it/biobakery4/metaphlan_databases/mpa_latest").getcode() != 200:
+            # use_zenodo = True
+            pass
     except:
         print('WARNING: It seems that you do not have Internet access.')
         if os.path.exists(os.path.join(bowtie2_db,'mpa_latest')):
@@ -266,20 +280,16 @@ def check_and_install_database(index, bowtie2_db, bowtie2_build, nproc, force_re
                 latest_db_version = [line.strip() for line in mpa_latest if not line.startswith('#')]
         else:
             print("""ERROR: Cannot find a local database. Please run MetaPhlAn using option "-x <database_name>".
-            You can download the MetaPhlAn database from \n {} \n {} \n {} 
-                  """.format('http://cmprod1.cibio.unitn.it/biobakery3/metaphlan_databases',ZENODO_DATABASE_DOWNLOAD, DROPBOX_DATABASE_DOWNLOAD))
+            You can download the MetaPhlAn database from \n {} 
+                  """.format('http://cmprod1.cibio.unitn.it/biobakery4/metaphlan_databases'))
             sys.exit()
 
     #try downloading from the segatalab website. If fails, use zenodo
     if index == 'latest':
-        if not use_zenodo:
-            mpa_latest = 'http://cmprod1.cibio.unitn.it/biobakery3/metaphlan_databases/mpa_latest'
-        else:
-            mpa_latest = 'https://zenodo.org/record/3957592/files/mpa_latest?download=1'
-
-        index = resolve_latest_database(bowtie2_db, mpa_latest, force_redownload_latest)
+        mpa_latest = 'http://cmprod1.cibio.unitn.it/biobakery4/metaphlan_databases/mpa_latest'
+        index = resolve_latest_database(bowtie2_db, mpa_latest, force_redownload_latest, offline)
     
-    if os.path.exists(os.path.join(bowtie2_db,'mpa_previous')):
+    if not offline and os.path.exists(os.path.join(bowtie2_db,'mpa_previous')):
         with open(os.path.join(bowtie2_db,'mpa_previous')) as mpa_previous:
             previous_db_version = ''.join([line.strip() for line in mpa_previous if not line.startswith('#')])
     
@@ -294,10 +304,12 @@ def check_and_install_database(index, bowtie2_db, bowtie2_build, nproc, force_re
                 
     if len(glob(os.path.join(bowtie2_db, "*{}*".format(index)))) >= 7:
         return index
-
+    if offline:
+        print("Database cannot be downloaded with the --offline option activated")
+        sys.exit()
     # download the tar archive and decompress
     sys.stderr.write("\nDownloading MetaPhlAn database\nPlease note due to "
                      "the size this might take a few minutes\n")
     download_unpack_tar(index, bowtie2_db, bowtie2_build, nproc, use_zenodo)
     sys.stderr.write("\nDownload complete\n")
-    return index
\ No newline at end of file
+    return index
