@@ -4,8 +4,8 @@ __author__ = ('Aitor Blanco Miguez (aitor.blancomiguez@unitn.it), '
               'Francesco Asnicar (f.asnicar@unitn.it), '
               'Moreno Zolfo (moreno.zolfo@unitn.it), '
               'Francesco Beghini (francesco.beghini@unitn.it)')
-__version__ = '4.0.1'
-__date__ = '24 Aug 2022'
+__version__ = '4.0.2'
+__date__ = '22 Sep 2022'
 
 import sys
 try:
@@ -43,8 +43,8 @@ def read_params():
     p = ap.ArgumentParser(description="", formatter_class=ap.ArgumentDefaultsHelpFormatter)
     p.add_argument('-d', '--database', type=str, default=DEFAULT_DATABASE,
                    help="The input MetaPhlAn database")
-    p.add_argument('-c', '--clade', type=str, default=None,
-                   help="The clades to investigate")
+    p.add_argument('-c', '--clades', type=str, nargs='+', default=[],
+                   help="The clades to investigate")   
     p.add_argument('-o', '--output_dir', type=str, default=None,
                    help="The output directory")
     
@@ -57,8 +57,8 @@ Checks the mandatory command line arguments of the script.
 :returns: the checked args
 """
 def check_params(args):
-    if not args.clade:
-        error('-c (or --clade) must be specified', exit=True, 
+    if len(args.clades) == 0:
+        error('-c (or --clades) must be specified', exit=True, 
             init_new_line=True)
     elif not args.output_dir:
         error('-o (or --output_dir) must be specified', exit=True, 
@@ -91,35 +91,33 @@ def check_dependencies():
 Extract the markers of a specific clade in a MetaPhlAn database
 
 :param database: the MetaPhlan markers database
-:param clade: the clade to extract markers
+:param clades: the clades to extract markers
 :param output_dir: the output directory
 :returns: the output file with the extracted sequences of the marker
 """
-def extract_markers(database, clade, output_dir):
-    info('\tGenerating DB markers FASTA...', init_new_line=True)
+def extract_markers(database, clades, output_dir):
+    info('\tExtracting markers from the Bowtie2 database...', init_new_line=True)
     fasta_markers = generate_markers_fasta(database, output_dir)
     info('\tDone.', init_new_line=True)
-    info('\tLoading MetaPhlan ' + __version__ + ' database...', init_new_line=True)
+    info('\tLoading MetaPhlAn {} database...'.format(database.split('/')[-1][:-4]), init_new_line=True)
     db = pickle.load(bz2.BZ2File(database))
     info('\tDone.',init_new_line=True)
-    markers = set([])
-    for marker in db['markers']:
-        species = db['markers'][marker]['clade']
-        if clade == species:
-            markers.add(marker)
-    if len(markers) == 0:
-        error("No markers were found for the clade \""+clade+"\" in the database", 
-            exit=True, init_new_line=True)
-    info('\tNumber of markers for the clade \"'+clade+"\": "+str(len(markers)), 
-        init_new_line=True)
-    output_file = output_dir+clade+".fna"
-    info('\tExporting markers...', init_new_line=True)
-    with open(output_file, 'w') as ofile:
-        for rec in SeqIO.parse(open(fasta_markers, 'r'), 'fasta'):
-            if rec.name in markers:
-                SeqIO.write(rec, ofile, 'fasta')
-    info('\tDone.', init_new_line=True)
-    
+    for clade in clades:
+        markers = set()
+        for marker in db['markers']:
+            if clade == db['markers'][marker]['clade']:
+                markers.add(marker)
+        if len(markers) == 0:
+            error('No markers were found for the clade "{}".'.format(clade), exit=False, init_new_line=True)
+        else:
+            info('\tNumber of markers for the clade "{}": {}'.format(clade, len(markers)), init_new_line=True)
+            output_file = output_dir+clade+".fna"
+            info('\tExporting markers for clade {}...'.format(clade), init_new_line=True)
+            with open(output_file, 'w') as ofile:
+                for rec in SeqIO.parse(open(fasta_markers, 'r'), 'fasta'):
+                    if rec.name in markers:
+                        SeqIO.write(rec, ofile, 'fasta')
+            info('\tDone.', init_new_line=True)    
     os.remove(fasta_markers)
     return output_file
 
@@ -137,7 +135,7 @@ def main():
     info("Start extract markers execution")
     # check_dependencies()
     args = check_params(args)
-    extract_markers(args.database, args.clade, args.output_dir)
+    extract_markers(args.database, args.clades, args.output_dir)
     exec_time = time.time() - t0
     info("Finish extract markers execution ("+str(round(exec_time, 2))+
         " seconds): Results are stored at \""+args.output_dir+"\"\n",
